<?php

/* Copyright (C) 2024 SuperAdmin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
require_once realpath(__DIR__ . '/..') . '/dolibarr.inc.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
require_once DOL_DOCUMENT_ROOT . '/product/class/html.formproduct.class.php';
dol_include_once('/ecotax/lib/ecotax.lib.php');

/**
 * Class ActionsEcotax
 */
class ActionsEcotax
{

	/**
	 * @var DoliDB Database handler.
	 */
	public $db;

	/**
	 * @var string Error code (or message)
	 */
	public $error = '';

	/**
	 * @var array Errors
	 */
	public $errors = array();

	/**
	 * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
	 */
	public $results = array();

	/**
	 * @var string String displayed by executeHook() immediately after return
	 */
	public $resprints;

	/**
	 * @var int		Priority of hook (50 is used if value is not defined)
	 */
	public $priority;

	/**
	 * Constructor
	 *
	 *  @param		DoliDB		$db      Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;
	}

	public function doActions($parameters, &$object, &$action)
	{
		global $conf;

		$contexts = explode(':', $parameters['context']);

		$allow_link_origin = (
			(in_array('invoicesuppliercard', $contexts)&& GETPOST('origin') == 'reception') // create invoice from reception card
			||
			(in_array('receptionlist', $contexts) && GETPOST('massaction') == 'confirm_createbills') // create invoices from reception list
		);
		if ($allow_link_origin) {
			$conf->global->ECOTAX_ALSO_ON_LINK_ORIGIN = 1;
		}
	}

	/**
	 * Execute action viewDictionaryFieldlist
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function viewDictionaryFieldlist($parameters, &$object, &$action)
	{
		if ($this->shouldEditOrDisplayEcotaxTypeDict($parameters, $object, $action)) {
			return $this->displayEcotaxTypeDict($parameters, $object, $action);
		}

		return 0;
	}

	/**
	 * Execute action createDictionaryFieldlist
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function createDictionaryFieldlist($parameters, &$object, &$action)
	{
		if ($this->shouldEditOrDisplayEcotaxTypeDict($parameters, $object, $action)) {
			return $this->editEcotaxTypeDict($parameters, $object, $action);
		}

		return 0;
	}

	/**
	 * Execute action editDictionaryFieldlist
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function editDictionaryFieldlist($parameters, &$object, &$action)
	{
		if ($this->shouldEditOrDisplayEcotaxTypeDict($parameters, $object, $action)) {
			return $this->editEcotaxTypeDict($parameters, $object, $action);
		}

		return 0;
	}

	private function shouldEditOrDisplayEcotaxTypeDict($parameters, &$object, &$action)
	{
		if ($parameters['tabname'] !== 'c_ecotax_type') {
			return false;
		}

		return true;
	}

	/**
	 * On Ecotax Dict Display Input line with a different behavior than standard
	 * @global type $db
	 * @global type $langs
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function editEcotaxTypeDict($parameters, &$object, &$action)
	{
		global $langs;
		$langs->load('ecotax@ecotax');
		$defaultCode = null;
		$defaultService = null;
		$defaultUnit = null;
		$defaultAmount = 0;
		$defaultSubjectToVAT = 0;
		if (!empty($object)) {
			$defaultCode = $object->code;
			$defaultService = $object->fk_service;
			$defaultUnit = $object->fk_unit;
			$defaultAmount = $object->unit_amount ?: 0;
			$defaultSubjectToVAT = $object->subject_to_vat;
		}

		$html = '
			<td class="">
				<input type="text" class="flat quatrevingtpercent"  maxlength="55" value="' . $defaultCode . '" name="code">
			</td>';

		$form = new Form($this->db);
		$formproduct = new FormProduct($this->db);

		$html .= '
			<td class="">';

		$html .= $form->select_produits(
				$defaultService, 'fk_service', 1, 0, 0, 1, 2, '', 0, array(), 0, '1', 0, '', 0, '', null, 1 // No Output
		);

		$html .= '
			</td>';

		$html .= '
			<td class="">';

		$html .= $formproduct->selectMeasuringUnits("fk_unit", "weight", $defaultUnit, '', 0);

		$html .= '
			</td>';

		$html .= '
			<td class="">
				<input  class="flat " value="' . $defaultAmount . '" name="unit_amount">
			</td>';

		$html .= '
			<td class="">';

		$checked = $defaultSubjectToVAT == 1 ? 'checked' : ' ';
		$html .= '<input type="checkbox" name="check_subject_to_vat_' . $action . '" id="check_subject_to_vat_' . $action . '" ' . $checked . ' value="1">';
		$html .= '<input type="hidden" name="subject_to_vat" id="subject_to_vat_' . $action . '" value="-1">';

		$html .= '
				<script type="module">
					import "' . dol_buildpath('ecotax/js/ecotax_dict_behavior.js', 3) . '?action=' . $action . '";
				</script> 
			</td>';

		print $html;

		return 1;
	}

	/**
	 * On Ecotax Dict Display View line with a different behavior than standard
	 * @global type $db
	 * @global type $langs
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function displayEcotaxTypeDict($parameters, &$object, &$action)
	{
		global $langs;

		$langs->load('ecotax@ecotax');

		$service = new Product($this->db);

		$retFetch = $service->fetch($object->fk_service);

		if ($retFetch < 0) {
			$this->error = $service->error;
			$this->errors = $service->errors;
		}

		$html = '
			<td class="">
				' . $object->code . '
			</td>
			<td class="">
				' . $service->getNomUrl() . '
			</td>
			<td class="">
				' . $langs->transnoentities(getDictionaryValue('c_units', 'label', $object->fk_unit)) . '
			</td>
			<td class="">
				' . ($object->unit_amount > 0 ? price($object->unit_amount) : "") . '
			</td>';

		$html .= '
			<td class="">';

		$html .= $object->subject_to_vat > 0 ? $langs->trans('Yes') : $langs->trans('No');

		$html .= '
			</td>';

		print $html;

		return 1;
	}

	/**
	 * Execute action formObjectOptions
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function formObjectOptions($parameters, &$object, &$action)
	{
		if ($this->shouldDisplayEcoTaxAmountExtrafield($parameters, $object, $action)) {
			return $this->displayEcoTaxAmountExtrafield($parameters, $object, $action);
		}

		return 0;
	}

	private function shouldDisplayEcoTaxAmountExtrafield($parameters, $object, $action)
	{
		if (!in_array('productcard', explode(':', $parameters['context']))) {
			return false;
		}

		if ($object->element != 'product') {
			return false;
		}

		if (empty($object->array_options["options_ecotax_ecotax"])) {
			return false;
		}

		if (!empty(getDictionaryValue('c_ecotax_type', 'unit_amount', $object->array_options["options_ecotax_ecotax"]))) {
			return false;
		}

		return true;
	}

	/**
	 * When a product has is extrafield "options_ecotax_ecotax" set to a value without "unit_amount" value set in
	 * dictionnary Ecotax_Type, the extrafield "options_ecotax_unit_amount" should be displayed.
	 * @global type $langs
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function displayEcoTaxAmountExtrafield($parameters, $object, $action)
	{
		global $langs;

		$langs->load('ecotax@ecotax');

		$out .= '
                <tr>';
		$extra_label = $langs->trans('EcotaxUnitAmount');

		$extra_value = $object->array_options['options_ecotax_unit_amount'];
		$href = $_SERVER["PHP_SELF"] . '?id=' . $object->id;

		if ($action == 'editextra_ecotax_unit_amount') {
			$out .= '
				<td>
					' . $extra_label . '
				</td>                    
				<td>
					<form enctype="multipart/form-data" 
						action="' . $href . '"   
						method="post" name="formextra">
						<input type="hidden" name="action" value="update_extras">
						<input type="hidden" name="attribute" value="ecotax_unit_amount">
						<input type="hidden" name="token" value="' . newToken() . '">
						<input name="options_ecotax_unit_amount" id="options_ecotax_unit_amount" size="6" value="' . $extra_value . '">
						<input type="submit" class="smallpaddingimp button" 
							value="' . dol_escape_htmltag($langs->trans('Modify')) . '">
						<input type="submit" name="cancel" class="smallpaddingimp button" 
							value="' . dol_escape_htmltag($langs->trans('Cancel')) . '">
					</form>
				</td>';
		} else {

			$out .= '
				<td>
					' . $extra_label . '
					<a class="editfielda reposition"
						href="' . $href . '&action=editextra_ecotax_unit_amount&token=' . newToken() . '">
						' . img_edit($langs->trans('Edit'), 1) . '
					</a>
				</td>
				<td>
					' . price($extra_value) . '€
				</td>';
		}

		$out .= '
			</tr>';

		print $out;

		return 0;
	}

	/**
	 * Execute action printObjectSubLine
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function printObjectSubLine($parameters, &$object, &$action)
	{
		if ($this->shouldDisplayEcotaxLine($parameters, $object, $action)) {
			return $this->displayEcoTaxLine($parameters, $object, $action);
		}

		return 0;
	}

	/**
	 * Execute action printObjectLine
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function printObjectLine($parameters, &$object, &$action)
	{
		if ($this->shouldDisplayEcotaxLine($parameters, $object, $action)) {
			return $this->displayEcoTaxLine($parameters, $object, $action);
		}

		return 0;
	}

	private function shouldDisplayEcotaxLine($parameters, $object, $action)
	{
		$handledContext = ['propalcard', 'ordercard', 'invoicecard', 'invoicereccard', 'invoicesuppliercard', 'ordersuppliercard'];
		if (!array_intersect($handledContext, explode(':', $parameters['context']))) {
			return false;
		}

		$handledLines = ['propaldet', 'commandedet', 'facturedet', 'facturedetrec', 'facture_fourn_det', 'commande_fournisseurdet'];
		if (!in_array($parameters['line']->element, $handledLines)) {
			return false;
		}

		if (empty($parameters['line']->fk_product)) {
			return false;
		}

		if (!isEcotaxLine($this->db, $parameters['line']->fk_product)) {
			$this->error = $this->db->lasterror();
			return false;
		}

		return true;
	}

	/**
	 * An Ecotax service line should not been edited moved or removed so button are hidden
	 * @global type $disableedit
	 * @global type $disablemove
	 * @global type $disableremove
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function displayEcoTaxLine($parameters, $object, $action)
	{
		global $disableedit, $disablemove, $disableremove;

		require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';
		$extrafields = new ExtraFields($this->db);
		$extrafields->fetch_name_optionals_label($parameters['table_element_line']);

		// This global are use to verify if buttons have to be displayed or not in printObjectLine function
		$disableedit = true;
		$disablemove = true;
		$disableremove = true;

		$object->printObjectLine(
			$action,
			$parameters['line'],
			'',
			$parameters['num'],
			$parameters['i'],
			$parameters['dateSelector'],
			$parameters['seller'],
			$parameters['buyer'],
			$parameters['selected'],
			$extrafields,
			$parameters['defaulttpldir']
		);

		$disableedit = false;
		$disablemove = false;
		$disableremove = false;

		return 1;
	}

	/**
	 * Execute action selectProductsListWhere
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function selectProductsListWhere($parameters, &$object, &$action)
	{
		if ($this->shouldFilterSelectProduct($parameters, $object, $action)) {
			return $this->filterSelectProduct($parameters, $object, $action);
		}

		return 0;
	}

	/**
	 * Execute action selectSuppliersProductsListWhere
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function selectSuppliersProductsListWhere($parameters, &$object, &$action)
	{
		if ($this->shouldFilterSelectProduct($parameters, $object, $action)) {
			return $this->filterSelectProduct($parameters, $object, $action);
		}

		return 0;
	}

	private function shouldFilterSelectProduct($parameters, $object, $action)
	{
		$handledContext = ['propalcard', 'ordercard', 'invoicecard', 'invoicereccard', 'invoicesuppliercard', 'ordersuppliercard'];
		if (!array_intersect($handledContext, explode(':', $parameters['context']))) {
			return false;
		}

		return true;
	}

	/**
	 * An ecotax service should not be added manually from the product selector so here is a condition to hide them
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function filterSelectProduct($parameters, $object, $action)
	{
		$this->resprints = '
			AND p.rowid NOT IN (SELECT fk_service FROM ' . $this->db->prefix() . 'c_ecotax_type)
			';

		return 0;
	}

	/**
	 * Execute action getNomUrl
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function getNomUrl($parameters, &$object, &$action)
	{
		if ($this->shouldDisplayEcotaxDifferent($parameters, $object, $action)) {
			return $this->displayEcotaxDifferent($parameters, $object, $action);
		}

		return 0;
	}

	private function shouldDisplayEcotaxDifferent($parameters, $object, $action)
	{
		$handledContext = ['propalcard', 'ordercard', 'invoicecard', 'invoicereccard', 'invoicesuppliercard', 'ordersuppliercard'];
		if (!array_intersect($handledContext, explode(':', $parameters['context']))) {
			return false;
		}

		if (empty($parameters['id'])) return false;

		if (!isEcotaxLine($this->db,$parameters['id'])) {
			$this->error = $this->db->lasterror();
			return false;
		}

		return true;
	}

	/**
	 * An ecotax Line should be identified clearly so we add a mention EcotaxLine
	 * @global type $langs
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function displayEcotaxDifferent($parameters, $object, $action)
	{
		global $langs;
		$langs->load('ecotax@ecotax');

		$this->resprints = ' <b>' . $langs->trans('EcotaxLine') . '</b>';

		return 0;
	}


	/**
	 * Execute action pdf_writelinedesc
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function pdf_writelinedesc($parameters, &$object, &$action, $hookmanager)
	{
		if ($this->shouldChangePdfBehavior($parameters, $object)) {
			return $this->changePdfLinesDesc($parameters, $object, $action);
		}

		return 0;
	}

	private function shouldChangePdfBehavior($parameters, $objectToDisplay)
	{
		if (!in_array('pdfgeneration', explode(':', $parameters['context']))) {
			return false;
		}

		if (!in_array($objectToDisplay->element, ['commande', 'propal', 'facture', 'order_supplier', 'invoice_supplier'])) {
			return false;
		}

		if (!isEcotaxLine($this->db,$objectToDisplay->lines[$parameters['i']]->fk_product)) {
			$this->error = $this->db->lasterror();
			return false;
		}

		return true;
	}

	/**
	 * An ecotax Line should be identified clearly so we add a mention EcotaxLine
	 * @global type $langs
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function changePdfLinesDesc($parameters, &$object, &$action)
	{
		global $langs;
		$langs->load('ecotax@ecotax');

		$labelproductservice = '<b>' . $langs->trans('EcotaxLine') . '</b> - ';
		$labelproductservice .= pdf_getlinedesc($object, $parameters['i'], $langs, 1, 0, 0);

		// Description
		$parameters['pdf']->writeHTMLCell(
			$parameters['w'],
			$parameters['h'],
			$parameters['posx'],
			$parameters['posy'],
			$langs->convToOutputCharset($labelproductservice),
			0,
			1,
			false,
			true,
			'J',
			true
		);

		$this->resprints = $labelproductservice;

		return 1;
	}

		/**
	 * Overloading the ODTSubstitution function : replacing the parent's function with the one below
	 *
	 * @param   array()         $parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             < 0 on error, 0 on success, 1 to replace standard code
	 */
	function ODTSubstitution($parameters, &$object, &$action, $hookmanager)
	{
		global $conf, $mysoc;

		$contexts = explode(':', $parameters['context']);

		if (in_array('odtgeneration', $contexts)) {
			$odfHandler = &$parameters['odfHandler'];
			$file = $parameters['file'];
			$obj = $parameters['object'];
			$outputlangs = $parameters['outputlangs'];
			$substitutionarray = &$parameters['substitutionarray'];

			if($obj->element === 'facture' || $obj->element === 'commande' || $obj->element === 'order_supplier' || $obj->element === 'propal')
			{
				$ecotaxtotal = $this->findEcotaxTotals($obj);

				$substitutionarray['object_total_ecotaxe_locale_dea'] = $ecotaxtotal['DEA'] ? price($ecotaxtotal['DEA']) : 0;
				$substitutionarray['object_total_ecotaxe_locale_deee'] = $ecotaxtotal['DEEE'] ? price($ecotaxtotal['DEEE']) : 0;
				$substitutionarray['object_total_ecotaxe_locale_tgap1'] = $ecotaxtotal['TGAP-1'] ? price($ecotaxtotal['TGAP-1']) : 0;
				$substitutionarray['object_total_ecotaxe_locale_tgap2'] = $ecotaxtotal['TGAP-2'] ? price($ecotaxtotal['TGAP-2']) : 0;
				$substitutionarray['object_total_ecotaxe_locale_tgap3'] = $ecotaxtotal['TGAP-3'] ? price($ecotaxtotal['TGAP-3']) : 0;
			}

		}

		return 0;
	}

	/**
	 * Execute action pdf_writelinedesc
	 *
	 * @param	array			$parameters		Array of parameters
	 * @param	CommonObject    $object         The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	string			$action      	'add', 'update', 'view'
	 * @return	int         					<0 if KO,
	 *                           				=0 if OK but we want to process standard actions too,
	 *                            				>0 if OK and we want to replace standard actions.
	 */
	public function afterPDFTotalTable($parameters, &$object, &$action, $hookmanager)
	{
		if ($this->shouldAddEcotaxAfterTotalTable($parameters, $object)) {
			return $this->addEcotaxAfterTotalTable($parameters, $object);
		}

		return 0;
	}

	private function shouldAddEcotaxAfterTotalTable($parameters, $object)
	{
		if (!in_array('pdfgeneration', explode(':', $parameters['context']))) {
			return false;
		}

		if (!in_array($parameters['object']->element, ['commande', 'propal', 'facture', 'order_supplier', 'invoice_supplier'])) {
			return false;
		}

		return true;
	}

	/**
	 * Show ecotax table above totals
	 * @global type $langs
	 * @param type $parameters
	 * @param type $object
	 * @param type $action
	 * @return int
	 */
	private function addEcotaxAfterTotalTable($parameters, &$object)
	{
		$langs = $parameters['outputlangs'];
		$langs->load('ecotax@ecotax');

		$ecotaxArray = $this->findEcotaxTotals($parameters['object']);

		$tab_top = $parameters['pdf']->GetY();

		if (empty($ecotaxArray)) {
			return $tab_top;
		}

		$html = '
		   <table>
			   <tr>
				   <td>' . $langs->transnoentities("IncludingEcotaxAmount") . '</td>
				   <td></td>
			   </tr>';

		foreach ($ecotaxArray as $ecotaxcode => $ecotaxtotal) {
			$html .= '
			   <tr>
				   <td>' . $langs->transnoentities("EcotaxLine") . ' ' . $ecotaxcode . ' HT</td>
				   <td style="text-align: right;">' . price($ecotaxtotal) . '</td>
			   </tr>';
		}

		$html .= '</table>';

		$posx = 120;
		$largcol = 80;

		$parameters['pdf']->SetXY($posx, $tab_top);
		$tab_hl = 4 * (count($ecotaxArray) + 1);

		$parameters['pdf']->writeHTMLCell($largcol, $tab_hl, '', '', $html, 0, 1);

		$parameters['index'] += count($ecotaxArray) + 1;

		return 1;
	}


	/**
	 * Function to determine ecotax total on object for each type
	 * @return bool
	 */
	protected function findEcotaxTotals($object)
	{
		global $db;

		$ecotaxArray = [];

		foreach ($object->lines as $line) {
			if (empty($line->fk_product)) {
				continue;
			}

			$sql = '
			   SELECT code
			   FROM ' . $db->prefix() . 'c_ecotax_type
			   WHERE fk_service=' . $line->fk_product;

			$resql = $db->query($sql);

			if (!$resql) {
				continue;
			}

			$num = $db->num_rows($resql);

			if (empty($num)) {
				continue;
			}

			$obj = $db->fetch_object($resql);

			if (empty($ecotaxArray[$obj->code])) {
				$ecotaxArray[$obj->code] = $line->total_ht;
			} else {
				$ecotaxArray[$obj->code] += $line->total_ht;
			}
		}

		return $ecotaxArray;
	}
}
